#include <stdio.h>
#include "global.h"
#include "mbuf.h"
#include "netuser.h"
#include "internet.h"
#include "timer.h"
#include "tcp.h"
#include "ip.h"
#include "trace.h"

/* TCP segment header flags */
static char *Tcpflags[] = {
	"FIN",	/* 0x01 */
	"SYN",	/* 0x02 */
	"RST",	/* 0x04 */
	"PSH",	/* 0x08 */
	"ACK",	/* 0x10 */
	"URG"	/* 0x20 */
};

/* Dump a TCP segment header. Assumed to be in network byte order */
void
tcp_dump(bpp,source,dest,check)
struct mbuf **bpp;
int32 source,dest;	/* IP source and dest addresses */
int check;		/* 0 if checksum test is to be bypassed */
{
	struct tcp seg;
	struct pseudo_header ph;
	int16 csum;

	if(bpp == NULLBUFP || *bpp == NULLBUF)
		return;

	/* Verify checksum */
	ph.source = source;
	ph.dest = dest;
	ph.protocol = TCP_PTCL;
	ph.length = len_mbuf(*bpp);
	csum = cksum(&ph,*bpp,ph.length);

	ntohtcp(&seg,bpp);

	printf("TCP: %u->%u Seq x%lx",seg.source,seg.dest,seg.seq,seg.ack);
	if(seg.flags.ack)
		printf(" Ack x%lx",seg.ack);
	if(seg.flags.urg)
		printf(" %s",Tcpflags[5]);
	if(seg.flags.ack)
		printf(" %s",Tcpflags[4]);
	if(seg.flags.psh)
		printf(" %s",Tcpflags[3]);
	if(seg.flags.rst)
		printf(" %s",Tcpflags[2]);
	if(seg.flags.syn)
		printf(" %s",Tcpflags[1]);
	if(seg.flags.fin)
		printf(" %s",Tcpflags[0]);

	printf(" Wnd %u",seg.wnd);
	if(seg.flags.urg)
		printf(" UP x%x",seg.up);
	/* Print options, if any */
	if(seg.mss != 0)
		printf(" MSS %u",seg.mss);
	if(check && csum != 0)
		printf(" CHECKSUM ERROR (%u)",csum);
	printf("\n");
}

